# copyright ################################# #
# This file is part of the BIHC Package.      #
# Copyright (c) CERN, 2024.                   #
# ########################################### #

'''
Impedance module to manage impedance object creation from a txt file exported 
from CST database or thanks to (i) the resisitive wall impedance model (for a 
circular pipe) or (ii) broad-band resonator model.

The created impedance consists in a list of numpy.ndarray: the first element is
the frequency vector, the second is the impedance.

* date: 12/12/2022
* author: Francesco Giordano, Elena de la Fuente, Leonardo Sito
'''

import numpy as np
import csv
import copy

from bihc.plot import Plot

class Impedance(Plot):
    """Define a Beam Coupling Impedance curve.

    It is possible to define an impedance curve using: (i) the broad-band 
    resonator model, (ii) the resistive wall impedance of a circular beam pipe, 
    and (iii) importing an external impedance text file (like the one that can
    be exported from CST).

    Mixin class for the Beam class.
    
    Parameters
    ----------
    f : numpy.ndarray, default np.linspace(0.1,2e9,int(1e5))
        Frequency array of the impedance curve.

        This is the numpy array containing the frequency point of the impedance
        curve. by default it is a linearly spaced vector going from 0.1 Hz to 2 
        GHz with 10'000 points.

    CST_file : str, deafult 0
        File name with extension of the txt file containing the impedance curve.
    
    Attributes
    ----------
    f : numpy.ndarray, default np.linspace(0.1,2e9,int(1e5))
        Frequency array of the impedance curve [Hz].
    Zr : numpy.ndarray
        Real part of the impedance in the speciefied frequency points [Ohm].
    Zi : numpy.ndarray
        Imaginary part of the impedance in the speciefied frequency points [Ohm].
    """   
    def __init__(self, f=np.linspace(0.1,2e9,int(1e5)), Z=None, CST_file=None):
    
        self.f = f
        self.Zr = np.zeros(len(f))
        self.Zi = np.zeros(len(f))

        if Z is not None:
            self.Zr = np.real(Z)
            self.Zi = np.imag(Z)
            if len(self.f) != len(self.Zr):
                print('[!] frequency array and impedance data have different lengths')

        self.isResonatorImpedance = False
        self.isRWImpedance = False

        if CST_file is not None:
            self.getImpedanceFromCST(CST_file)     

    def __add__(self, Zn):
        Z = Impedance(self.f)

        if not np.array_equal(self.f, Zn.f):
            Z.Zr = np.interp(self.f, Zn.f, Zn.Zr)
            Z.Zi = np.interp(self.f, Zn.f, Zn.Zi)
        else:
            Z.Zr = Zn.Zr
            Z.Zi = Zn.Zi

        Z.Zr += self.Zr
        Z.Zi += self.Zi   

        return Z
        
    def getResonatorImpedance(self,Rs,Qr,fr):
        """Creating the impedance curve from the broad-band resonator model.

        This methods creates an impedance curve using the broad-band resonator 
        model. It requires a shunt impedance value, a quality factor value, and
        the resonant frequency value. 

        Parameters
        ----------
        Rs : float
            Shunt impedance in the broad-band resonator model [Ohm].
        Qr : float
            Quality factor in the broad-band resonator model.
        fr : float
            Resonant frequency in the broad-band resonator model [Hz].
        
        Returns
        -------
        [f, Z] : numpy.ndarray list 
            Impedance curve. Returns the list of numpy arrays [frequency, 
            complex impedance].
        """

        self.fr = fr
        self.Rs = Rs
        self.Qr = Qr
        self.isResonatorImpedance = True

        f = self.f
        mask1 = f == 0
        f[mask1] = 1e-5
        Z = Rs/(1+ 1j*Qr*(f/fr - fr/f))
        
        self.Zr = np.real(Z)
        self.Zi = np.imag(Z)
        self.Z = Z
        
        return [self.f, self.Zr+1j*self.Zi]
        
    def getRWImpedance(self, L ,b, sigma, f=np.linspace(0.1,2e9,int(1e5))):
        """Creating the impedance curve from the resistive wall impedance model.

        This methods creates an impedance curve using the resistive wall 
        impedance model for a resistive pipe of circular section with. 
        It considers the thick wall regime.

        Parameters
        ----------
        L : float
            Length of the pipe in the resistive wall impedance model [m].                
        b : float
            Pipe radius in the resisitve wall impedance model [m].
        sigma : float
            Electrical conductivity of the pipe in the resistive wall
            impedance model [Siemens/m].
        
        Returns
        -------
        [f, Z] : numpy.ndarray list 
            Impedance curve. Returns the list of numpy arrays [frequency, 
            complex impedance].
        """
        Z0 = 376.73
        c = 3e8
        f = self.f
        
        # !n.b. problem with the signum
        Z = (1+1j*np.sign(f))*L/(2*np.pi*b)*np.sqrt(Z0*np.abs(2*np.pi*f)/(2*c*sigma))
        
        self.Zr = np.real(Z)
        self.Zi = np.imag(Z)
        self.Z = Z

        self.b = b
        self.sigma = sigma
        self.L = L
        self.isRWImpedance = True
        
        return [self.f, self.Zr+1j*self.Zi]
        
    def getImpedanceFromCST(self, path, unit='GHz', skip_header=2, skip_footer=0):
        """Creating the impedance curve from CST file.

        This methods creates an impedance curve using a txt file, usually
        exported from CST.

        Parameters
        ----------
        path : str
            File name with extension.                
        unit : str, default "GHz"
            Units of the frquency array from the simulation. Either GHz or MHz.
        skip_header : int, default 2
            Line index to skip to at the beginning of the txt file.
        skip_footer : int, default 0
            Line index from which to skip to at the end of the txt file.
        
        Returns
        -------
        [f, Z] : numpy.ndarray list 
            Impedance curve. Returns the list of numpy arrays [frequency, 
            complex impedance].
        """   
        data = np.genfromtxt(path, skip_header=skip_header, skip_footer=skip_footer)
        
        if unit == 'GHz':
            self.f= data[:,0]*1e9
        elif unit == 'MHz':
            self.f= data[:,0]*1e6
  
        self.Zr = data[:,1]
        try:
            self.Zi = data[:,2]
        except:
            self.Zi = np.zeros_like(self.Zr)

        self.Z = self.Zr+1j*self.Zi

        return [self.f, self.Z]
    
    def getImpedanceFromFunc(self, func, f=None):
        '''Gets impedance from a python function
        in the form of Z = func(f)

        Ensures compatibility to Xwakes wake objects
        https://github.com/xsuite/xwakes/

        Example:
        -------
        >>> import xwakes as xw
        >>> wake = xw.WakeResonator(r=1e8, q=10, f_r=1e9,
                                    kind='longitudinal')
        >>> import bihc
        >>> Z = bihc.Impedance(f=np.linspace(0,1e9,10000))
        >>> Z.getImpedanceFromFunc(wake.components[0].impedance)
        '''

        if f is not None:
            self.f = f
        
        self.Z = func(f)
        self.Zr = np.real(self.Z)
        self.Zi = np.imag(self.Z)

        return [self.f, self.Z]

    def getImpedanceFromPandas(self, path, unit='GHz'):
        import pandas as pd

        data = pd.read_csv(path)
        data.columns = ['f', 'Zr']
        
        if unit == 'GHz':
            data['f']= data['f']*1e9
        elif unit == 'MHz':
            data['f']= data['f']*1e6
  
        self.f = data['f']
        self.Zr = data['Zr']
        
        return data
    
    def copy(self):
        obj = type(self).__new__(self.__class__)
        obj.__dict__.update(self.__dict__)
        return obj

    def getFrequencyRegions(self, vlines=None, figsize=[12,6], dpi=200):
        '''Interactively select points 
        of the impedance curve. The list of
        points will be returned and saved in 
        `self.freqregions`. How to use:
        - place cursor + Spacebar --> pick
        - Delete --> unpick
        - Enter --> End picking

        Returns
        -------
        freqregions: list
            np.array of frequencies of picked points [Hz]
        '''
        import matplotlib.pyplot as plt

        fig, ax = plt.subplots(figsize=figsize, dpi=dpi)
        ax.plot(self.f*1e-9, self.Zr, color='k')

        if vlines is not None:
            ax.vlines(vlines*1e-9, ymin=np.min(self.Zr), ymax=np.max(self.Zr), color='k', linestyle='dashed', alpha=0.4)

        ax.set_ylabel('Impedance [$\Omega$]')
        ax.set_xlabel('Frequency [GHz]')
        ax.set_title('Spacebar --> pick | Delete --> unpick | Enter --> Finish', color='red')
        fig.tight_layout()

        #get frequencies 
        points = plt.ginput(n=0, timeout=0, mouse_add=None, mouse_pop=None, mouse_stop=None)
        self.freqregions = np.array(points)[:,0]*1e9
        plt.close()   

        return self.freqregions




# copyright ################################# #
# This file is part of the BIHC Package.      #
# Copyright (c) CERN, 2024.                   #
# ########################################### #

from . import power
from . import plot
from . import impedance
from . import beam


from .beam import Beam
from .impedance import Impedance

from .plot import progressbar

from ._version import __version__


# copyright ################################# #
# This file is part of the BIHC Package.      #
# Copyright (c) CERN, 2024.                   #
# ########################################### #

'''
User defined filling schemes defined as
a sequence of true/falses. A True indicates
de presence of a bunch and False indicates
an empty slot
'''

# SPS user defined filling general
def fillingSchemeSPS(nbunches, ntrains, batchspacing=9):
    '''
    Returns the filling scheme for the SPS

    Parameters
    ----------
    ntrains: number of injections (batches)
    '''
    # Define filling scheme: parameters
    # ntrains = 4 # number of trains/batches
    nslots = 924 # Defining total number of slots for SPS
    # nbunches = 72 # Defining a number of bunchs e.g. 18, 36, 72.. 
    # batchspacing = 9 # Batch spacing in 25 ns slots 45/5

    # Defining the trains as lists of True/Falses
    bt = [True]*nbunches
    st = [False]*batchspacing
    sc = [False]*(nslots - (nbunches+batchspacing)*ntrains)
    an = (bt + st)*ntrains + sc

    return an

# LHC user defined filling general
def fillingSchemeLHC(ninj, nbunches, ntrains, batchspacing=7, injspacing=37):
    '''
    Returns the filling scheme for the LHC 
    using the standard pattern: 

    "nbunches x ntrains for ninj"

    Parameters
    ----------
    ninj: int
        number of injections of the "ntrains x nbunches" scheme
    nbunches: optional, default 72
        number of bunches per train
    ntrains: optional, default 4
        number of trains of bunches per single injection
    '''

    # Define filling scheme: parameters
    # ninj = 14 # Defining number of injections
    nslots = 3564 # Defining total number of slots for LHC
    # ntrain = 4 # Defining the number of trains
    # nbunches = 72 # Defining a number of bunchs e.g. 18, 36, 72.. 
    batchS = batchspacing # Batch spacing in 25 ns slots
    injspacing = injspacing # Injection spacing in 25 ns slots

    # Defining the trains as lists of True/Falses
    bt = [True]*nbunches
    st = [False]*batchS
    stt = [False]*injspacing
    sc = [False]*(nslots-(ntrains*nbunches*ninj+((ntrains-1)*(batchS)*ninj)+((1)*injspacing*(ninj))))
    an1 = (bt+st)*ntrains + stt
    an = an1 * ninj + sc # This is the final true false sequence that is the beam distribution

    return an

# -------------------------------------
# Case-specific filling schemes:

def fillingSchemeSPS_standard(ntrains):
    '''
    Returns standard the filling scheme for the SPS

    Parameters
    ----------
    ntrains: number of trains (batches)
    '''
    # Define filling scheme: parameters
    nslots = 920 # Defining total number of slots for SPS
    nbunches = 72 # Defining a number of bunchs e.g. 18, 36, 72.. 
    batchspacing = 9 # Batch spacing in 25 ns slots 45/5

    # Defining the trains as lists of True/Falses
    bt = [True]*nbunches
    st = [False]*batchspacing
    sc = [False]*(nslots - (nbunches+batchspacing)*ntrains)
    an = (bt + st)*ntrains + sc

    return an

def fillingSchemeSPS_BCMS(ninj):
    '''
    Returns the filling scheme for the SPS

    Parameters
    ----------
    ninj: number of injections (batches)
    nbunches: default, 48. number of bunches per train
    '''
    # Define filling scheme: parameters
    ntrain = 1 # SPS has 1 train per cycle
    nslots = 920 # Defining total number of slots for SPS
    nbunches = 48 # Defining a number of bunchs e.g. 18, 36, 72.. 
    batchspacing = 8 # Batch spacing in 25 ns slots (200 ns)

    # Defining the trains as lists of True/Falses
    bt = [True]*nbunches
    st = [False]*batchspacing
    sc = [False]*(nslots - (nbunches+batchspacing)*ninj)
    an = (bt + st)*ninj + sc

    return an

def fillingSchemeSPS_8b4e(ninj):
    '''
    Returns the filling scheme for the SPS 
    using the 8b4e pattern

    Parameters
    ----------
    ninj: number of injections (batches)
    '''
    # Define filling scheme: parameters
    nslots = 920 # Defining total number of slots for SPS
    nbunches = 8*7 # Defining number of bunches e.g. 18, 36, 72.. 
    nempty = 4*6  #Defining number of empty slots between bunches
    batchspacing = 8 # Batch spacing in 25 ns slots (200 ns)

    # Defining the trains as lists of True/Falses
    bt = ([True]*8+[False]*4)*6+[True]*8
    st = [False]*batchspacing
    sc = [False]*(nslots - (nbunches+nempty+batchspacing)*ninj)
    an = (bt + st)*ninj + sc

    return an

def fillingSchemeLHC_standard(ninj, nbunches=72, ntrains=4):
    '''
    Returns the filling scheme for the LHC 
    using the standard pattern: 

    "nbunches x ntrains for ninj"

    Parameters
    ----------
    ninj: int
        number of injections of the "ntrains x nbunches" scheme
    nbunches: optional, default 72
        number of bunches per train
    ntrains: optional, default 4
        number of trains of bunches per single injection
    '''

    # Define filling scheme: parameters
    ninj = ninj # Defining number of injections
    nslots = 3564 # Defining total number of slots for LHC
    ntrain = ntrains # Defining the number of trains
    nbunches = nbunches # Defining a number of bunchs e.g. 18, 36, 72.. 
    batchS = 7 # Batch spacing in 25 ns slots
    injspacing = 37 # Injection spacing in 25 ns slots

    # Defining the trains as lists of True/Falses
    bt = [True]*nbunches
    st = [False]*batchS
    stt = [False]*injspacing
    sc = [False]*(nslots-(ntrain*nbunches*ninj+((ntrain-1)*(batchS)*ninj)+((1)*injspacing*(ninj))))
    an1 = (bt+st)*ntrains + stt
    an = an1 * ninj + sc # This is the final true false sequence that is the beam distribution

    return an

# SPS user defined filling scheme: AWAKE single bunch
def fillingSchemeSPS_AWAKE(nbunches=1, ntrains=1):
    '''
    Returns the filling scheme for the SPS

    Parameters
    ----------
    ntrains: number of injections (batches)
    '''
    # Define filling scheme: parameters
    #ntrains = 4 # number of trains/batches
    nslots = 924 # Defining total number of slots for SPS
    #nbunches = 72 # Defining a number of bunchs e.g. 18, 36, 72.. 
    batchspacing = 9 # Batch spacing in 25 ns slots 45/5

    # Defining the trains as lists of True/Falses
    bt = [True]*nbunches
    st = [False]*batchspacing
    sc = [False]*(nslots - (nbunches+batchspacing)*ntrains)
    an = (bt + st)*ntrains + sc

    return an


# copyright ################################# #
# This file is part of the BIHC Package.      #
# Copyright (c) CERN, 2024.                   #
# ########################################### #

'''
Power module to manage power loss computations
for one beam and two beams case.

Power loss is computed for a given beam (or beams)
power spectrum and the specified impedance map in
frequency.

* date: 12/12/2022
* author: Francesco Giordano, Elena de la Fuente, Leonardo Sito
'''

import numpy as np
from tqdm import tqdm

class Power():
    '''Power Mixin class

    Class to encapsulate power computation methods
    It is inherited by Beam class
    '''
    def getPloss(self,Z):
        '''
        Computes the power loss for a given impedance object
        Implemented by Francesco Giordano

        Parameters
        ----------
        Z : object
            Impedance object returned by Impedance class with 
            the frequency information of the impedance map given 
            by the user
        '''

        e=1.621e-19
        t=self.longitudinalProfile[0]
        f0=1/(t[-1]-t[0])
        [f,S]=self.spectrum

        Zreal=Z.Zr
        Zf=Z.f

        if np.max(f)>np.max(Z.f):
            mask1=f>=0
            mask2=f<=np.max(Z.f)
            mask=mask1*mask2
            f=f[mask]
            Zreal=np.interp(f,Z.f,Z.Zr)
            S=S[mask]
            
        elif np.max(f)<np.max(Z.f):
            mask1=Z.f>=0
            mask2=Z.f<=np.max(f)
            mask=mask1*mask2
            Zf=Z.f[mask]
            Zreal=Z.Zr[mask]
            mask3= f>=0
            f=f[mask3]
            S=S[mask3]
            
            Zreal=np.interp(f,Zf,Zreal)
    
        mask3= f>=0
        f=f[mask3]
        S=S[mask3]   
        Zreal=Zreal[mask3]
        Zf=f

        P=f0*e*S*self.filledSlots*self.Np
        P_density=2*(P**2)*Zreal
        P_loss=np.sum(P_density) 
        if self.verbose:
            print(f'Computed Power loss: {P_loss} W') 

        return P_loss, P_density;

    def getShiftedPloss(self, Z, shift=20e6):
        '''
        Computes the power loss, shifting the impedance 
        curve rigidly in steps given by `shift`, to overlap
        with the spectral lines, giving a best (away from the
        line) and worst (on top of the line) case scenario.

        Parameters
        ----------
        Z : object
            Impedance object returned by Impedance class with 
            the frequency information of the impedance map given 
            by the user
        shift : float
            Frequency shift to be applied in steps to the 
            impedance curve
        '''

        [f,S] = self.spectrum
        deltaF = f[1]-f[0]
        fmax = Z.f[-1]

        #if impedance file is too short, we zero padd it
        #Otherwise the interpolation will assume for the 
        #missing frequencies a constant value 

        Zint = np.interp(f, Z.f, Z.Zr)
        Zmod = Z.copy()
        Zmod.f, Zmod.Zr = f, Zint

        if Zmod.f[-1] > fmax: 
            mask = Zmod.f > fmax
            Zmod.Zr[mask] = 0.0

        size = int(shift/deltaF)
        shifts = np.arange(-size, size, 1, dtype=int) #shifting every frev

        power = np.array([])
        for step in tqdm(shifts, "Computing scan: ", total=2*size):
            Zmod.Zr = np.roll(Zint, step) 
            if step > 0: Zmod.Zr[:step] = 0.0
            if step < 0: Zmod.Zr[:2*size-step] = 0.0
            power = np.append(power, self.getPloss(Zmod)[0])

        Zmod.f, Zmod.Zr, Zmod.Zi = f, Zint, Zint*0.
        self.Z = Zmod.copy()
        Zmod.Zr = np.roll(Zint, shifts[np.argmax(power)])
        self.Zmax = Zmod.copy()

        return shifts, power

    def getShiftedPowerSpectrum(self, Z, shift=20e6):
        '''
        Computes the power loss spectrum, shifting the impedance 
        curve rigidly in steps given by `shift`, to overlap
        with the spectral lines, giving a best (away from the
        line) and worst (on top of the line) case scenario.

        Parameters
        ----------
        Z : object
            Impedance object returned by Impedance class with 
            the frequency information of the impedance map given 
            by the user
        shift : float
            Frequency shift to be applied in steps to the 
            impedance curve
        '''

        [f,S] = self.spectrum
        deltaF = f[1]-f[0]
        fmax = Z.f[-1]

        #if impedance file is too short, we zero padd it
        #Otherwise the interpolation will assume for the 
        #missing frequencies a constant value 

        Zint = np.interp(f, Z.f, Z.Zr)
        Zmod = Z.copy()
        Zmod.f, Zmod.Zr = f, Zint

        if Zmod.f[-1] > fmax: 
            mask = Zmod.f > fmax
            Zmod.Zr[mask] = 0.0

        size = int(shift/deltaF)
        shifts = np.arange(-size, size, 1, dtype=int) #shifting every frev

        power_spectrum = []
        for step in tqdm(shifts, "Computing scan: ", total=2*size):
            Zmod.Zr = np.roll(Zint, step) 
            if step > 0: Zmod.Zr[:step] = 0.0
            if step < 0: Zmod.Zr[:2*size-step] = 0.0
            power_spectrum.append(self.getPloss(Zmod)[1]) #1: spectrum!

        power_spectrum = np.array(power_spectrum)
        Zmod.f, Zmod.Zr, Zmod.Zi = f, Zint, Zint*0.
        self.Z = Zmod.copy()

        return shifts, power_spectrum
    
    def get2BeamPloss(self, Z_0, tau_s=None, s=None, offset1=None, offset2=None, Z_1=None):
        '''
        Computes the power loss for the two beams case
        given impedance object and the pahse_shift between 
        the two beams

        Implemented by Francesco Giordano

        Parameters
        ----------
        Z_0 : object
            Impedance object returned by Impedance class with 
            the frequency information of the impedance map given 
            by the user
        tau_s : float list
            Phase shift values between the two beams in seconds [s]
        s : float list
            Distances from the interaction point in [m]
        '''

        self.s = s
        if tau_s is not None:
            self.tau_s = tau_s
        elif s is not None:
            self.tau_s = 2*s/c
        else: 
            raise Exception('Specify s (distance from IP) or tau_s (phase shift of the two beams)')

        e = 1.621e-19
        t = self.longitudinalProfile[0]
        f0 = 1/(t[-1]-t[0])
        [f,S] = self.spectrum
        Zreal_0 = Z_0.Zr
        Zf_0 = Z_0.f

        #Z_0
        if np.max(f)>np.max(Z_0.f):
            mask1=f>=0
            mask2=f<=np.max(Z_0.f)
            mask=mask1*mask2
            f=f[mask]
            Zreal_0=np.interp(f,Z_0.f,Z_0.Zr)
            S=S[mask]
            
        elif np.max(f)<np.max(Z_0.f):
            mask1=Z_0.f>=0
            mask2=Z_0.f<=np.max(f)
            mask=mask1*mask2

            Zf_0 = Z_0.f[mask]
            Zreal_0 = Z_0.Zr[mask]
            Zreal_1 = Z_1.Zr[mask]
            mask3 = f>=0
            f = f[mask3]
            S = S[mask3]
            
            Zreal_0 = np.interp(f,Zf_0,Zreal_0)

        #Z_1
        if Z_1 is not None:
            Zreal_1 = Z_1.Zr
            Zf_1 = Z_1.f

            if np.max(f)>np.max(Z_1.f):
                mask1=f>=0
                mask2=f<=np.max(Z_1.f)
                mask=mask1*mask2
                f=f[mask]
                Zreal_1=np.interp(f,Z_1.f,Z_1.Zr)
                S=S[mask]
                
            elif np.max(f)<np.max(Z_1.f):
                mask1=Z_0.f>=0
                mask2=Z_0.f<=np.max(f)
                mask=mask1*mask2

                Zf_1 = Z_1.f[mask]
                Zreal_1 = Z_1.Zr[mask]
                mask3 = f>=0
                f = f[mask3]
                S = S[mask3]

                Zreal_1 = np.interp(f,Zf_1,Zreal_1)
        
            mask3 = f>=0
            Zreal_1 = Zreal_1[mask3]
            Zf_1 = f

        mask3 = f>=0
        f = f[mask3]
        S = S[mask3]   
        Zreal_0 = Zreal_0[mask3]
        Zf_0 = f

        if Z_1 is not None and (offset1 is not None or offset2 is not None):

            if offset2 is None:
                offset2 = np.zeros_like(offset1)
            if offset1 is None:
                offset1 = np.zeros_like(offset2)

            # Formula with beam offsets
            P = (2*f0*e*self.filledSlots*self.Np*S)**2
            P_loss = []
            P_density_list = []
            for i, shift in tqdm(enumerate(tau_s), "Computing 2-beam power with offset: "):
                P_density = P*(Zreal_0+(offset1[i]+offset2[i])*Zreal_1)*(1-np.cos(2*np.pi*f*shift))
                P_loss.append(np.sum(P_density))  
        else:
            # Simplified formula
            P = (2*f0*e*self.filledSlots*self.Np*S)**2
            P_loss = []
            P_density_list = []
            for shift in tqdm(tau_s, "Computing 2-beam power: "):
                P_density = P*Zreal_0*(1-np.cos(2*np.pi*f*shift))
    #           P_density_list.append(P_density)
                P_loss.append(np.sum(P_density))  

        if self.verbose:
            pass
            #print(f'Computed Power loss: {P_loss} W') 

        self.P_loss = P_loss
        return P_loss



# copyright ################################# #
# This file is part of the BIHC Package.      #
# Copyright (c) CERN, 2024.                   #
# ########################################### #

from setuptools import setup, find_packages
from pathlib import Path

# read version
version_file = Path(__file__).parent / 'bihc/_version.py'
dd = {}
with open(version_file.absolute(), 'r') as fp:
    exec(fp.read(), dd)
__version__ = dd['__version__']

# read long_description
long_description = (Path(__file__).parent / "README.md").read_text(encoding="utf-8")

#########
# Setup #
#########

setup(
    name="bihc",
    version=__version__,
    description="BIHC: Beam-Induced Heating Computation package",
    author="Elena de la Fuente, Francesco Giordano, Leonardo Sito",
    author_email="elena.de.la.fuente.garcia@cern.ch", 
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    url="https://github.com/ImpedanCEI/BIHC",
    project_urls={"Bug Tracker": "https://github.com/ImpedanCEI/BIHC/issues"},
    install_requires = [
                    'numpy',
                    'matplotlib',
                    'scipy',
                       ],
    classifiers=[
        "Development Status :: 4 - Beta",
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        "Operating System :: OS Independent",
        "Intended Audience :: Science/Research",
		"Topic :: Scientific/Engineering :: Physics",
    ],

)
